@@ -38,21 +38,24 @@ module Main where
 import Network.URI
     ( URI(..), URIAuth(..)
     , nullURI
+    , rectify, rectifyAuth
     , parseURI, parseURIReference, parseRelativeReference, parseAbsoluteURI
     , parseAbsoluteURI
     , isURI, isURIReference, isRelativeReference, isAbsoluteURI
     , uriIsAbsolute, uriIsRelative
     , relativeTo, nonStrictRelativeTo
     , relativeFrom
-    , uriToString
+    , uriToString, uriAuthToString
     , isUnescapedInURIComponent
     , isUnescapedInURI, escapeURIString, unEscapeString
     , normalizeCase, normalizeEscape, normalizePathSegments
+    , pathSegments
     )
 
 import Test.HUnit
 
 import Data.Maybe (fromJust)
+import Data.List (intercalate)
 import System.IO (openFile, IOMode(WriteMode), hClose)
 import qualified Test.Framework as TF
 import qualified Test.Framework.Providers.HUnit as TF
@@ -133,6 +136,7 @@ testURIRef026 = testURIRef AbsId "http://[::FFFF:129.144.52.38]:80/index.html"
 testURIRef027 = testURIRef AbsId "http://[2010:836B:4179::836B:4179]"
 testURIRef028 = testURIRef RelRf "//[2010:836B:4179::836B:4179]"
 testURIRef029 = testURIRef InvRf "[2010:836B:4179::836B:4179]"
+testURIRef030 = testURIRef AbsId "http://[fe80::ff:fe00:1%25eth0]"
 -- RFC2396 test cases
 testURIRef031 = testURIRef RelRf "./aaa"
 testURIRef032 = testURIRef RelRf "../aaa"
@@ -1259,6 +1263,72 @@ testIsRelative = TF.testGroup "testIsRelative"
   , TF.testCase "testIsRelative04" $ testUriIsRelative "?what=that"
   ]
 
+testPathSegmentsRoundTrip :: URI -> Assertion
+testPathSegmentsRoundTrip u =
+  let segs = pathSegments u
+
+      dropSuffix _suf []              = []
+      dropSuffix suf [x] | suf == x   = []
+                         | otherwise = [x]
+      dropSuffix suf (x:xs)          = x : dropSuffix suf xs
+
+      dropPrefix _pre []                 = []
+      dropPrefix pre (x:xs) | pre == x   = xs
+                            | otherwise = (x:xs)
+      strippedUriPath = dropSuffix '/' $ dropPrefix '/' $ uriPath u
+  in
+     (Data.List.intercalate "/" segs @?= strippedUriPath)
+
+assertJust _f Nothing = assertFailure "URI failed to parse"
+assertJust f  (Just x) = f x
+
+testPathSegments = TF.testGroup "testPathSegments"
+  [ TF.testCase "testPathSegments03" $
+        assertJust testPathSegmentsRoundTrip $ parseURIReference ""
+  , TF.testCase "testPathSegments04" $
+        assertJust testPathSegmentsRoundTrip $ parseURIReference "/"
+  , TF.testCase "testPathSegments05" $
+        assertJust testPathSegmentsRoundTrip $ parseURIReference "//"
+  , TF.testCase "testPathSegments06" $
+        assertJust testPathSegmentsRoundTrip $ parseURIReference "foo//bar/"
+  , TF.testCase "testPathSegments07" $
+        assertJust testPathSegmentsRoundTrip $ parseURIReference "/foo//bar/"
+  , TF.testCase "testPathSegments03" $
+        assertJust testPathSegmentsRoundTrip $ parseURI "http://example.org"
+  , TF.testCase "testPathSegments04" $
+        assertJust testPathSegmentsRoundTrip $ parseURI "http://example.org/"
+  , TF.testCase "testPathSegments05" $
+        assertJust testPathSegmentsRoundTrip $ parseURI "http://example.org//"
+  , TF.testCase "testPathSegments06" $
+        assertJust testPathSegmentsRoundTrip $ parseURI "http://ex.ca/foo//bar/"
+  , TF.testCase "testPathSegments07" $
+        assertJust testPathSegmentsRoundTrip $ parseURI "http://ex.ca/foo//bar/"
+  ]
+
+testRectify = TF.testGroup "testRectify"
+  [ TF.testCase "" $ testEq "testRectify"
+    (show $ rectify $ URI { uriScheme = "http" ,
+                            uriAuthority = Just (URIAuth "ezra" "www.google.com" "80") ,
+                            uriPath = "/foo/bar" ,
+                            uriQuery = "foo=bar&baz=quz" ,
+                            uriFragment = "chap10" })
+    "http://ezra@www.google.com:80/foo/bar?foo=bar&baz=quz#chap10"
+  , -- According to RFC2986, any URL without a // does not have an authority component.
+    -- Therefore tag: URIs have all their content in the path component. This is supported
+    -- by the urn: example in section 3. Note that tag: URIs have no leading slash on their
+    -- path component.
+    TF.testCase "" $ testEq "testRectify"
+    "tag:timothy@hpl.hp.com,2001:web/externalHome"
+    (show $ rectify $ URI { uriScheme = "tag" ,
+                            uriAuthority = Nothing,
+                            uriPath = "timothy@hpl.hp.com,2001:web/externalHome",
+                            uriQuery = "" ,
+                            uriFragment = "" })
+  , TF.testCase "" $ testEq "testRectifyAuth"
+    "//ezra@www.google.com:80"
+    ((uriAuthToString id . Just . rectifyAuth $ URIAuth "ezra" "www.google.com" "80") "")
+  ]
+
 -- Full test suite
 allTests =
   [ testURIRefSuite
@@ -1274,6 +1344,8 @@ allTests =
   , testAltFn
   , testIsAbsolute
   , testIsRelative
+  , testPathSegments
+  , testRectify
   ]
 
 main = TF.defaultMain allTests
